﻿/*
		This is the main object for i-weather AS3 library
		Please visit http://www.i-weather.org for more detailed information

		Copyright (c) 2010 fabric | ch
		Author Christian Babski
		Version   1.0,  20.06.2010

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
*/
package ch.fabric.iweather {

	import flash.display.MovieClip;
	import flash.text.TextField;
	import flash.geom.ColorTransform;
	import flash.utils.Timer;
	import flash.events.Event;
	import flash.events.TimerEvent;

	public class IWeather {
	
		// DO NOT EDIT THESE VALUES IN ORDER TO STAY I-WEATHER COMPATIBLE
		private static const _dayDuration:Number = 90460000;
		// private static const _startTime:Number = 1004090400000; // 26.10.2001 10:00:00 GMT (UTC) from -> 26.10.2001 to -> 12.08.2010
		private static const _startTime:Number = 1004072400000; // 26.10.2001 05:00:00 GMT (UTC) from -> 12.08.2010
		// --------------------------------------------------------------
		
		private var _currentDay:Number;
		
		private var _currentHour:uint;
		private var _currentMinute:uint;
		private var _currentSecond:uint;
		private var _currentR:uint;
		private var _currentG:uint;
		private var _currentB:uint;
		private var _currentT:uint = 0;
		
		private var _registeredText:Array = new Array();
		private var _registeredTimeText:Array = new Array();
		private var _registeredMovieClip:Array = new Array();
		
		private var _updateTimer:Timer;
		private var _flashTimer:Timer = null;
		
		private var _flashR:uint;
		private var _flashG:uint;
		private var _flashB:uint;
		private var _flashCount:uint;
		
		// DO NOT EDIT THESE VALUES IN ORDER TO STAY I-WEATHER COMPATIBLE
		private var _colors:Array = new Array(
				new Array(getTimeInSeconds("00:00:00"),255,215,  0,102),
				new Array(getTimeInSeconds("00:30:00"),255,215,  0,102),
				new Array(getTimeInSeconds("04:10:05"),255,255,  0,102),
				new Array(getTimeInSeconds("05:10:05"),155,255, 55,102),
				new Array(getTimeInSeconds("06:10:05"),  0,255,255,102),
				new Array(getTimeInSeconds("12:03:50"), 70,235,255,102),
				new Array(getTimeInSeconds("13:03:50"), 70,235,255,102),
				new Array(getTimeInSeconds("18:57:35"),  0,255,255,102),
				new Array(getTimeInSeconds("19:57:35"),155,255, 55,102),
				new Array(getTimeInSeconds("20:57:35"),255,255,  0,102),
				new Array(getTimeInSeconds("24:37:40"),255,215,  0,102),
				new Array(getTimeInSeconds("25:07:40"),255,215,  0,102));
		// --------------------------------------------------------------
		
		// Twinkle control (associated set/get function)

		// Allow to switch on/off twinkle function
		private var _iweatherTwinkle:Boolean = true;
		
		// Determine the number of twinkle
		private var _iweatherTwinkleCount:uint = 0;
		
		// Determine the twinkle's frequency
		private var _iweatherTwinkleDelay:uint = 0;
		
		// Allow to change iweather mode
		// 0 - is the real-time mode
		// 1 - is the user-time mode (allow to display a given moment in the i-weather day)
		private var _iwMode:int = 0;
		
		private var _simStart:int = 0;
		private var _simDuration:int = Math.floor(_dayDuration/1000.0);
		private var _simStop:int = _simStart + _simDuration;
		private var _simIncr:int = 1;
		private var _simTimeLeft:int = _simDuration;
		
		// ---------------
		
		public function IWeather() {
			_updateTimer = new Timer(1000);
			// _updateTimer.addEventListener(TimerEvent.TIMER, EL_updateTime);
			
			updateTime();
		}
		
		public function addText(tf:TextField):void {
			if (tf != null)
				_registeredText.push(tf);
		}
		public function addTimeText(tf:TextField):void {
			if (tf != null)
				_registeredTimeText.push(tf);
		}
		public function addMovieClip(mc:MovieClip):void {
			if (mc != null)
				_registeredMovieClip.push(mc);
		}
		
		public function start():void {
			trace("[(ASC) IWeather start()]");
			switch(_iwMode) {
				case 0:
					_updateTimer.addEventListener(TimerEvent.TIMER, EL_updateTime);
					break;
					
				case 1:
					_updateTimer.addEventListener(TimerEvent.TIMER, EL_updateUserTime);
					break;
			}
			
			_updateTimer.start();
			
			if (_iweatherTwinkle) {
				if (_flashTimer == null) {
					if (_iweatherTwinkleDelay > 0)
						_flashTimer = new Timer(_iweatherTwinkleDelay, 1);
					else
						_flashTimer = new Timer(1000+Math.round(Math.random()*30000), 1);
					_flashTimer.addEventListener(TimerEvent.TIMER, EL_doFlash);
				} else {
					if (_iweatherTwinkleDelay == 0)
						_flashTimer.delay = 1000+Math.round(Math.random()*30000);
				}
				_flashTimer.start();
			}
		}
		public function stop():void {
			trace("[(ASC) IWeather stop()]");
			_updateTimer.stop();
			_updateTimer.reset();
			
			switch(_iwMode) {
				case 0:
					_updateTimer.removeEventListener(TimerEvent.TIMER, EL_updateTime);
					break;
					
				case 1:
					_updateTimer.removeEventListener(TimerEvent.TIMER, EL_updateUserTime);
					break;
			}
			
			if (_flashTimer != null) {
				_flashTimer.stop();
				_flashTimer.reset();
			}
		}
		private function EL_updateTime(evt:TimerEvent):void {
			// trace("[(ASC) IWeather EL_updateTime()]");
			updateTime();
			updateGraphicalElements();
		}
		private function EL_updateUserTime(evt:TimerEvent):void {
			// trace("[(ASC) IWeather EL_updateUserTime()]");
			updateUserTime();
			updateGraphicalElements();
		}
		private function updateGraphicalElements():void {
			var i:int;
			
			for(i=0;i<_registeredText.length;i++) {
				_registeredText[i].textColor = (_currentT << 16) | (_currentT << 8) | _currentT;
			}
			for(i=0;i<_registeredTimeText.length;i++) {
				_registeredTimeText[i].text = getCurrentDayString()+" days | "+getCurrentHourString()+":"+getCurrentMinuteString()+":"+getCurrentSecondString()+" i-weather time"; // +getDayDurationString();
				_registeredTimeText[i].textColor = (_currentT << 16) | (_currentT << 8) | _currentT;
			}
			for(i=0;i<_registeredMovieClip.length;i++) {
				var colorTransform:ColorTransform = _registeredMovieClip[i].transform.colorTransform;
				colorTransform.color = (_currentR << 16) | (_currentG << 8) | _currentB;
				_registeredMovieClip[i].transform.colorTransform = colorTransform;
			}
		}
		
		final private function updateTime():void {
			var initTime:Number;
			var pct:Number;

			initTime = (new Date()).getTime() - _startTime;
			_currentDay = (int) (initTime/_dayDuration);
			
			_currentHour = (int) ((initTime - _currentDay*_dayDuration) / 3600000);
			_currentMinute = (int) ((initTime - _currentDay*_dayDuration - _currentHour*3600000) / 60000);
			_currentSecond = (int) ((initTime - _currentDay*_dayDuration - _currentHour*3600000 - _currentMinute*60000) / 1000);
			
			var i:int = 1;
			var cT:int = (int)((initTime - _currentDay*_dayDuration) / 1000);
			
			while (_colors[i][0] < cT)
				i++;
			
			pct = (cT - _colors[i-1][0]) / (_colors[i][0] - _colors[i-1][0]);
			
			_currentR = Math.round(_colors[i-1][1] + (_colors[i][1]-_colors[i-1][1]) * pct);
			_currentG = Math.round(_colors[i-1][2] + (_colors[i][2]-_colors[i-1][2]) * pct);
			_currentB = Math.round(_colors[i-1][3] + (_colors[i][3]-_colors[i-1][3]) * pct);
						
			// trace("[(AS) IWeather upateTime()] - "+i+" - "+_currentR+" * "+_currentG+" * "+_currentB+" - "+cT+" - "+pct);
			
			_currentT = _colors[i-1][4] + Math.round((_colors[i][4]-_colors[i-1][4]) * pct);
		}

		final private function updateUserTime():void {
			var initTime:Number;
			var pct:Number;
			
			initTime = _simStart * 1000;
			_currentDay = 0;
			
			_currentHour = (int) (_simTimeLeft / 3600);
			_currentMinute = (int) ((_simTimeLeft - _currentDay*_dayDuration - _currentHour*3600) / 60);
			_currentSecond = (int) ((_simTimeLeft - _currentDay*_dayDuration - _currentHour*3600 - _currentMinute*60));
			
			var i:int = 1;
			var cT:int = (int)((initTime - _currentDay*_dayDuration) / 1000);
			
			while (_colors[i][0] < cT)
				i++;
			
			pct = (cT - _colors[i-1][0]) / (_colors[i][0] - _colors[i-1][0]);
			
			_currentR = _colors[i-1][1] + Math.round((_colors[i][1]-_colors[i-1][1]) * pct);
			_currentG = _colors[i-1][2] + Math.round((_colors[i][2]-_colors[i-1][2]) * pct);
			_currentB = _colors[i-1][3] + Math.round((_colors[i][3]-_colors[i-1][3]) * pct);
			
			_currentT = _colors[i-1][4] + Math.round((_colors[i][4]-_colors[i-1][4]) * pct);

			_simStart += _simIncr;
			_simTimeLeft -= _simIncr;
			if (_simStart > _simStop) {
				_currentHour = 0;
				_currentMinute = 0;
				_currentSecond = 0;
				stop();
			}
		}

		final private function EL_doFlash(evt:TimerEvent):void {
			_flashR = _currentR;
			_flashG = _currentG;
			_flashB = _currentB;
			
			if (_iweatherTwinkleCount > 0)
				_flashCount = _iweatherTwinkleCount;
			else
				_flashCount = 5 + Math.round(Math.random()*50);
			for(var i=0;i<_registeredMovieClip.length;i++) {
				_registeredMovieClip[i].addEventListener(Event.ENTER_FRAME, EL_flash);
				
				// Stop, reset and re-innitialise the next flash/twinkles timer
				_flashTimer.stop();
				_flashTimer.reset();
				if (_iweatherTwinkleDelay == 0)
					_flashTimer.delay = 5000+Math.round(Math.random()*30000);
				_flashTimer.start();
			}
		}
		final private function EL_flash(evt:Event):void {
			var colorTransform:ColorTransform;
			var fVR, fVG, fVB:int;
			
			if ((_flashCount%2) == 0) {
				colorTransform = evt.currentTarget.transform.colorTransform;
				fVR = _flashR+_flashCount;
				fVG = _flashG+_flashCount;
				fVB = _flashB+_flashCount;
				if (fVR > 255) fVR = 255;
				if (fVG > 255) fVG = 255;
				if (fVB > 255) fVB = 255;
				
				colorTransform.color = (fVR << 16) | (fVG << 8) | fVB;
				evt.currentTarget.transform.colorTransform = colorTransform;
			} else {
				colorTransform = evt.currentTarget.transform.colorTransform;
				fVR = _flashR-_flashCount;
				fVG = _flashG-_flashCount;
				fVB = _flashB-_flashCount;
				if (fVR < 0) fVR = 0;
				if (fVG < 0) fVG = 0;
				if (fVB < 0) fVB = 0;
				
				colorTransform.color = (fVR << 16) | (fVG << 8) | fVB;
				evt.currentTarget.transform.colorTransform = colorTransform;
			}

			if (evt.currentTarget == _registeredMovieClip[0]) {
				_flashCount--;
			}
				
			if (_flashCount == 0) {
				evt.currentTarget.removeEventListener(Event.ENTER_FRAME, EL_flash);
			}
		}	

		final public function globalMode():void { _iwMode = 0; }
		final public function userMode():void { _iwMode = 1; }
		final public function userSimParameter(simStart:int, simIncr:int, simDuration:int):void {
			_iwMode = 1;
			if (simStart < 0)
				_simStart = 0;
			else if (simStart > Math.floor(_dayDuration/1000.0))
				_simStart = Math.floor(_dayDuration/1000.0);
			else
				_simStart = simStart;
			if (simDuration > Math.floor(_dayDuration/1000.0))
				_simStop = Math.floor(_dayDuration/1000.0);
			else
				_simStop = simStart + simDuration;
				
			_simIncr = simIncr;
			_simTimeLeft = simDuration;
		}

		final public function getCurrentDay():Number { return _currentDay; }
		final public function getCurrentDayString():String {
			var currentD:String = ""+_currentDay;
			while (currentD.length < 6)
				currentD = "0"+currentD;
			return currentD;
		}
		final public function getCurrentHour():uint { return _currentHour; }
		final public function getCurrentHourString():String {
			var currentH:String = ""+_currentHour;
			if (_currentHour < 10)
				currentH = "0"+currentH;
			return currentH;
		}
		final public function getCurrentMinute():uint { return _currentMinute; }
		final public function getCurrentMinuteString():String {
			var currentM:String = ""+_currentMinute;
			if (_currentMinute < 10)
				currentM = "0" + currentM;
			return currentM;
		}
		final public function getCurrentSecond():uint { return _currentSecond; }
		final public function getCurrentSecondString():String {
			var currentS:String = ""+_currentSecond;
			if (_currentSecond < 10)
				currentS = "0" + currentS;
			return currentS;
		}
		
		// Real time access function to know the current state of i-weather (time and color)
		final public function getCurrentR():uint { return _currentR; }
		final public function getCurrentG():uint { return _currentG; }
		final public function getCurrentB():uint { return _currentB; }
		final public function getCurrentC():uint { return 255-_currentR; }
		final public function getCurrentM():uint { return 255-_currentG; }
		final public function getCurrentY():uint { return 255-_currentB; }
		final public function getCurrentT():uint { return _currentT; }
		final public function getDayDurationString():String { return "25:07:40"; }
				
		// Activate/Desactivate twinkles. Twinkles are activated by default
		final public function twinkleActivation(onOff:Boolean):void { _iweatherTwinkle = onOff; }
		// Set twinkle frequency, every 'value' seconds, twinkle will occurre, 0 means random
		final public function twinkleFrequency(value:uint):void { if (value > 0) _iweatherTwinkleDelay = value*1000; }
		// Set number of vibration during one twinkle, higher is longer, 0 means random
		final public function twinkleNumber(value:uint):void { _iweatherTwinkleCount = value; }
		// ----------------------------------------------------------------------------------
		
		// Absolute function to get i-weather data (e.g. what color at a given moment in the i-weather day)
		
		// Return the associated color at the cyclePercent of the i-weather day
		// Return a 7 elements array containing R,G,B,T,H,M,S
		final public function getCycleColor(cyclePercent:Number):Array {
			var res:Array = new Array(-1,-1,-1,-1,-1,-1,-1);
			
			if (cyclePercent >= 0.0 && cyclePercent <= 100.0) {
				var desiredTime:int = Math.floor(cyclePercent*_dayDuration/100.0/1000.0);
				var i:int = 1;

				while (_colors[i][0] < desiredTime)
					i++;
								
				var pct:Number = (desiredTime - _colors[i-1][0]) / (_colors[i][0] - _colors[i-1][0]);
				
				res[0] = _colors[i-1][1] + Math.round((_colors[i][1]-_colors[i-1][1]) * pct);
				res[1] = _colors[i-1][2] + Math.round((_colors[i][2]-_colors[i-1][2]) * pct);
				res[2] = _colors[i-1][3] + Math.round((_colors[i][3]-_colors[i-1][3]) * pct);
							
				res[3] = _colors[i-1][4] + Math.round((_colors[i][4]-_colors[i-1][4]) * pct);
					
				res[4] = Math.floor(desiredTime/3600);
				res[5] = Math.floor((desiredTime - res[4]*3600)/60);
				res[6] = desiredTime - res[4]*3600 - res[5] * 60;
			}
			
			return res;
		}
		
		// Return the associated color at a given hour (valid format to submit is hh:mm:ss) of the i-weather day (hh:mm:ss)
		// Return a 7 elements array containing R,G,B,T,H,M,S
		final public function getHourColor(hour:String):Array {
			var res:Array = new Array(-1,-1,-1,-1,-1,-1,-1);
			
			if (hour.indexOf(":") == 2 && hour.lastIndexOf(":") == 5) {
				var h:int = int(hour.substring(0,2));
				var m:int = int(hour.substring(3,5));
				var s:int = int(hour.substring(6,8));
				var desiredTime:int = h*3600 + m*60 + s;

				if (desiredTime*1000 <= _dayDuration) {
					
					var i:int = 1;
					
					while(_colors[i][0] < desiredTime)
						i++;
									
					var pct:Number = (desiredTime - _colors[i-1][0]) / (_colors[i][0] - _colors[i-1][0]);
					
					res[0] = _colors[i-1][1] + Math.round((_colors[i][1]-_colors[i-1][1]) * pct);
					res[1] = _colors[i-1][2] + Math.round((_colors[i][2]-_colors[i-1][2]) * pct);
					res[2] = _colors[i-1][3] + Math.round((_colors[i][3]-_colors[i-1][3]) * pct);
								
					res[3] = _colors[i-1][4] + Math.round((_colors[i][4]-_colors[i-1][4]) * pct);
					
					res[4] = h;
					res[5] = m;
					res[6] = s;
				}
			}
			
			return res;
		}
		
		final private function getTimeInSeconds(time:String):int {
			return (int(time.substring(0,2))*3600 + int(time.substring(3,5))*60 + int(time.substring(6,8)));
		}
	}
}
